@@ -1,15 +1,16 @@
+from django.core.exceptions import ValidationError
 from django.db import models
 from django.urls import reverse
 
 from nautobot.dcim.fields import ASNField
 from nautobot.dcim.models import CableTermination, PathEndpoint
-from nautobot.extras.models import ObjectChange, RelationshipModel, StatusModel
+from nautobot.extras.models import ObjectChange, StatusModel
 from nautobot.extras.utils import extras_features
 from nautobot.core.fields import AutoSlugField
-from nautobot.core.models.generics import BaseModel, OrganizationalModel, PrimaryModel
-from nautobot.utilities.utils import serialize_object
+from nautobot.core.models.generics import OrganizationalModel, PrimaryModel
+from nautobot.utilities.utils import serialize_object, serialize_object_v2
+
 from .choices import CircuitTerminationSideChoices
-from .querysets import CircuitQuerySet
 
 
 __all__ = (
@@ -17,9 +18,61 @@ __all__ = (
     "CircuitTermination",
     "CircuitType",
     "Provider",
+    "ProviderNetwork",
 )
 
 
+@extras_features(
+    "custom_fields",
+    "custom_links",
+    "custom_validators",
+    "export_templates",
+    "graphql",
+    "relationships",
+    "webhooks",
+)
+class ProviderNetwork(PrimaryModel):
+    name = models.CharField(max_length=100, db_index=True)
+    slug = AutoSlugField(populate_from="name")
+    provider = models.ForeignKey(to="circuits.Provider", on_delete=models.PROTECT, related_name="provider_networks")
+    description = models.CharField(max_length=200, blank=True)
+    comments = models.TextField(blank=True)
+
+    csv_headers = [
+        "provider",
+        "name",
+        "slug",
+        "description",
+        "comments",
+    ]
+
+    class Meta:
+        ordering = ("provider", "name")
+        constraints = (
+            models.UniqueConstraint(fields=("provider", "name"), name="circuits_providernetwork_provider_name"),
+        )
+        unique_together = ("provider", "name")
+
+    def __str__(self):
+        return self.name
+
+    def get_absolute_url(self):
+        return reverse("circuits:providernetwork", args=[self.slug])
+
+    def to_csv(self):
+        return (
+            self.provider.name,
+            self.name,
+            self.slug,
+            self.description,
+            self.comments,
+        )
+
+    @property
+    def display(self):
+        return f"{self.provider} {self.name}"
+
+
 @extras_features(
     "custom_fields",
     "custom_links",
@@ -43,7 +96,7 @@ class Provider(PrimaryModel):
         verbose_name="ASN",
         help_text="32-bit autonomous system number",
     )
-    account = models.CharField(max_length=30, blank=True, verbose_name="Account number")
+    account = models.CharField(max_length=100, blank=True, verbose_name="Account number")
     portal_url = models.URLField(blank=True, verbose_name="Portal URL")
     noc_contact = models.TextField(blank=True, verbose_name="NOC contact")
     admin_contact = models.TextField(blank=True, verbose_name="Admin contact")
@@ -154,7 +207,23 @@ class Circuit(PrimaryModel, StatusModel):
     description = models.CharField(max_length=200, blank=True)
     comments = models.TextField(blank=True)
 
-    objects = CircuitQuerySet.as_manager()
+    # Cache associated CircuitTerminations
+    termination_a = models.ForeignKey(
+        to="circuits.CircuitTermination",
+        on_delete=models.SET_NULL,
+        related_name="+",
+        editable=False,
+        blank=True,
+        null=True,
+    )
+    termination_z = models.ForeignKey(
+        to="circuits.CircuitTermination",
+        on_delete=models.SET_NULL,
+        related_name="+",
+        editable=False,
+        blank=True,
+        null=True,
+    )
 
     csv_headers = [
         "cid",
@@ -200,30 +269,34 @@ class Circuit(PrimaryModel, StatusModel):
             self.comments,
         )
 
-    def _get_termination(self, side):
-        for ct in self.terminations.all():
-            if ct.term_side == side:
-                return ct
-        return None
-
-    @property
-    def termination_a(self):
-        return self._get_termination("A")
-
-    @property
-    def termination_z(self):
-        return self._get_termination("Z")
-
 
 @extras_features(
+    "custom_fields",
+    "custom_links",
     "custom_validators",
+    "export_templates",
     "graphql",
     "relationships",
+    "statuses",
+    "webhooks",
 )
-class CircuitTermination(BaseModel, PathEndpoint, CableTermination, RelationshipModel):
+class CircuitTermination(PrimaryModel, PathEndpoint, CableTermination):
     circuit = models.ForeignKey(to="circuits.Circuit", on_delete=models.CASCADE, related_name="terminations")
     term_side = models.CharField(max_length=1, choices=CircuitTerminationSideChoices, verbose_name="Termination")
-    site = models.ForeignKey(to="dcim.Site", on_delete=models.PROTECT, related_name="circuit_terminations")
+    site = models.ForeignKey(
+        to="dcim.Site",
+        on_delete=models.PROTECT,
+        related_name="circuit_terminations",
+        blank=True,
+        null=True,
+    )
+    provider_network = models.ForeignKey(
+        to="circuits.ProviderNetwork",
+        on_delete=models.PROTECT,
+        related_name="circuit_terminations",
+        blank=True,
+        null=True,
+    )
     port_speed = models.PositiveIntegerField(verbose_name="Port speed (Kbps)", blank=True, null=True)
     upstream_speed = models.PositiveIntegerField(
         blank=True,
@@ -240,9 +313,22 @@ class CircuitTermination(BaseModel, PathEndpoint, CableTermination, Relationship
         unique_together = ["circuit", "term_side"]
 
     def __str__(self):
-        return "Side {}".format(self.get_term_side_display())
+        return f"Termination {self.term_side}: {self.site or self.provider_network}"
+
+    def get_absolute_url(self):
+        return reverse("circuits:circuittermination", args=[self.pk])
+
+    def clean(self):
+        super().clean()
+
+        # Must define either site *or* provider network
+        if self.site is None and self.provider_network is None:
+            raise ValidationError("A circuit termination must attach to either a site or a provider network.")
+        if self.site and self.provider_network:
+            raise ValidationError("A circuit termination cannot attach to both a site and a provider network.")
 
     def to_objectchange(self, action):
+
         # Annotate the parent Circuit
         try:
             related_object = self.circuit
@@ -254,8 +340,9 @@ class CircuitTermination(BaseModel, PathEndpoint, CableTermination, Relationship
             changed_object=self,
             object_repr=str(self),
             action=action,
-            related_object=related_object,
             object_data=serialize_object(self),
+            object_data_v2=serialize_object_v2(self),
+            related_object=related_object,
         )
 
     @property
