@@ -211,14 +211,18 @@ class ObjectListView(ObjectPermissionRequiredMixin, View):
         # Check for YAML export support
         elif "export" in request.GET and hasattr(model, "to_yaml"):
             response = HttpResponse(self.queryset_to_yaml(), content_type="text/yaml")
-            filename = "nautobot_{}.yaml".format(self.queryset.model._meta.verbose_name_plural)
+            filename = "{}{}.yaml".format(
+                settings.BRANDING_PREPENDED_FILENAME, self.queryset.model._meta.verbose_name_plural
+            )
             response["Content-Disposition"] = 'attachment; filename="{}"'.format(filename)
             return response
 
         # Fall back to built-in CSV formatting if export requested but no template specified
         elif "export" in request.GET and hasattr(model, "to_csv"):
             response = HttpResponse(self.queryset_to_csv(), content_type="text/csv")
-            filename = "nautobot_{}.csv".format(self.queryset.model._meta.verbose_name_plural)
+            filename = "{}{}.csv".format(
+                settings.BRANDING_PREPENDED_FILENAME, self.queryset.model._meta.verbose_name_plural
+            )
             response["Content-Disposition"] = 'attachment; filename="{}"'.format(filename)
             return response
 
@@ -243,13 +247,22 @@ class ObjectListView(ObjectPermissionRequiredMixin, View):
         }
         RequestConfig(request, paginate).configure(table)
 
+        filter_form = None
+        if self.filterset_form:
+            if request.GET:
+                # Bind form to the values specified in request.GET
+                filter_form = self.filterset_form(request.GET, label_suffix="")
+            else:
+                # Use unbound form with default (initial) values
+                filter_form = self.filterset_form(label_suffix="")
+
         context = {
             "content_type": content_type,
             "table": table,
             "permissions": permissions,
             "action_buttons": self.action_buttons,
             "table_config_form": TableConfigForm(table=table),
-            "filter_form": self.filterset_form(request.GET, label_suffix="") if self.filterset_form else None,
+            "filter_form": filter_form,
         }
         context.update(self.extra_context())
 
