@@ -1,3 +1,5 @@
+import json
+
 from django.contrib.auth import get_user_model
 from django.test import override_settings
 from django.urls import reverse
@@ -48,7 +50,7 @@ from nautobot.dcim.models import (
     VirtualChassis,
 )
 from nautobot.extras.models import ConfigContextSchema, SecretsGroup, Status
-from nautobot.ipam.models import VLAN
+from nautobot.ipam.models import IPAddress, VLAN
 from nautobot.utilities.testing import APITestCase, APIViewTestCases
 from nautobot.virtualization.models import Cluster, ClusterType
 
@@ -1266,6 +1268,24 @@ class DeviceTest(APIViewTestCases.APIViewTestCase):
         )
         self.assertHttpStatus(response, status.HTTP_400_BAD_REQUEST)
 
+    def test_patching_primary_ip4_success(self):
+        """
+        Validate we can set primary_ip4 on a device using a PATCH.
+        """
+        # Add object-level permission
+        self.add_permissions("dcim.change_device")
+
+        dev = Device.objects.get(name="Device 3")
+        dev_intf = Interface.objects.create(name="Ethernet1", device=dev, type="1000base-t")
+        dev_ip_addr = IPAddress.objects.create(address="192.0.2.1/24", assigned_object=dev_intf)
+
+        patch_data = {"primary_ip4": dev_ip_addr.pk}
+
+        response = self.client.patch(
+            self._get_detail_url(Device.objects.get(name="Device 3")), patch_data, format="json", **self.header
+        )
+        self.assertHttpStatus(response, status.HTTP_200_OK)
+
 
 class ConsolePortTest(Mixins.ComponentTraceMixin, APIViewTestCases.APIViewTestCase):
     model = ConsolePort
@@ -1989,6 +2009,56 @@ class VirtualChassisTest(APIViewTestCases.APIViewTestCase):
             "domain": "newdomain",
         }
 
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+    def test_null_master(self):
+        """Test setting the virtual chassis master to null."""
+        url = reverse("dcim-api:virtualchassis-list")
+        response = self.client.get(url + "?name=Virtual Chassis 1", **self.header)
+        self.assertHttpStatus(response, status.HTTP_200_OK)
+        virtual_chassis_1 = response.json()["results"][0]
+
+        # Make sure the master is set
+        self.assertNotEqual(virtual_chassis_1["master"], None)
+
+        # Set the master of Virtual Chassis 1 to null
+        url = reverse("dcim-api:virtualchassis-detail", kwargs={"pk": virtual_chassis_1["id"]})
+        payload = {"name": "Virtual Chassis 1", "master": None}
+        self.add_permissions(f"{self.model._meta.app_label}.change_{self.model._meta.model_name}")
+        response = self.client.patch(url, data=json.dumps(payload), content_type="application/json", **self.header)
+
+        # Make sure the master is now null
+        self.assertHttpStatus(response, status.HTTP_200_OK)
+        self.assertEqual(response.json()["master"], None)
+
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+    def test_remove_chassis_from_master_device(self):
+        """Test removing the virtual chassis from the master device."""
+        url = reverse("dcim-api:virtualchassis-list")
+        response = self.client.get(url + "?name=Virtual Chassis 1", **self.header)
+        self.assertHttpStatus(response, status.HTTP_200_OK)
+        self.assertEqual(response.json()["count"], 1)
+        virtual_chassis_1 = response.json()["results"][0]
+
+        # Make sure the master is set
+        self.assertNotEqual(virtual_chassis_1["master"], None)
+
+        master_device = Device.objects.get(pk=virtual_chassis_1["master"]["id"])
+
+        # Set the virtual_chassis of the master device to null
+        url = reverse("dcim-api:device-detail", kwargs={"pk": master_device.id})
+        payload = {
+            "device_type": str(master_device.device_type.id),
+            "device_role": str(master_device.device_role.id),
+            "site": str(master_device.site.id),
+            "status": "active",
+            "virtual_chassis": None,
+        }
+        self.add_permissions("dcim.change_device")
+        response = self.client.patch(url, data=json.dumps(payload), content_type="application/json", **self.header)
+
+        # Make sure deletion attempt failed
+        self.assertHttpStatus(response, status.HTTP_400_BAD_REQUEST)
+
 
 class PowerPanelTest(APIViewTestCases.APIViewTestCase):
     model = PowerPanel
