@@ -1,16 +1,15 @@
 import logging
-from collections import OrderedDict
 
 from django import forms
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
+from django.core.exceptions import ValidationError
 from django.core.serializers.json import DjangoJSONEncoder
-from django.core.validators import ValidationError
 from django.db import models
 from django.db.models import Q
 
 from nautobot.extras.choices import RelationshipTypeChoices, RelationshipSideChoices
-from nautobot.extras.utils import FeatureQuery
+from nautobot.extras.utils import FeatureQuery, extras_features
 from nautobot.extras.models import ChangeLoggedModel
 from nautobot.core.fields import AutoSlugField
 from nautobot.core.models import BaseModel
@@ -53,7 +52,7 @@ class RelationshipModel(models.Model):
         related_query_name="destination_%(app_label)s_%(class)s",  # e.g. 'destination_dcim_rack'
     )
 
-    def get_relationships(self, include_hidden=False):
+    def get_relationships(self, include_hidden=False, advanced_ui=None):
         """
         Return a dictionary of queryset for all custom relationships
 
@@ -74,6 +73,9 @@ class RelationshipModel(models.Model):
             }
         """
         src_relationships, dst_relationships = Relationship.objects.get_for_model(self)
+        if advanced_ui is not None:
+            src_relationships = src_relationships.filter(advanced_ui=advanced_ui)
+            dst_relationships = dst_relationships.filter(advanced_ui=advanced_ui)
         content_type = ContentType.objects.get_for_model(self)
 
         sides = {
@@ -82,9 +84,9 @@ class RelationshipModel(models.Model):
         }
 
         resp = {
-            RelationshipSideChoices.SIDE_SOURCE: OrderedDict(),
-            RelationshipSideChoices.SIDE_DESTINATION: OrderedDict(),
-            RelationshipSideChoices.SIDE_PEER: OrderedDict(),
+            RelationshipSideChoices.SIDE_SOURCE: {},
+            RelationshipSideChoices.SIDE_DESTINATION: {},
+            RelationshipSideChoices.SIDE_PEER: {},
         }
         for side, relationships in sides.items():
             for relationship in relationships:
@@ -122,43 +124,43 @@ class RelationshipModel(models.Model):
 
         return resp
 
-    def get_relationships_data(self):
+    def get_relationships_data(self, advanced_ui=None):
         """
         Return a dictionary of relationships with the label and the value or the queryset for each.
 
         Returns:
             response {
                 "source": {
-                    <relationship #1>: {
+                    <relationship #1>: {   # one-to-one relationship that self is the source of
                         "label": "...",
                         "peer_type": <ContentType>,
                         "has_many": False,
-                        "value": <model>,
+                        "value": <model>,     # single destination for this relationship
                         "url": "...",
                     },
-                    <relationship #2>: {
+                    <relationship #2>: {   # one-to-many or many-to-many relationship that self is a source for
                         "label": "...",
                         "peer_type": <ContentType>,
                         "has_many": True,
                         "value": None,
-                        "queryset": <queryset #2>
+                        "queryset": <queryset #2>   # set of destinations for the relationship
                     },
                 },
                 "destination": {
-                    (same format as source)
+                    (same format as source - relationships that self is the destination of)
                 },
                 "peer": {
-                    (same format as source)
+                    (same format as source - symmetric relationships that self is involved in)
                 },
             }
         """
 
-        relationships_by_side = self.get_relationships()
+        relationships_by_side = self.get_relationships(advanced_ui=advanced_ui)
 
         resp = {
-            RelationshipSideChoices.SIDE_SOURCE: OrderedDict(),
-            RelationshipSideChoices.SIDE_DESTINATION: OrderedDict(),
-            RelationshipSideChoices.SIDE_PEER: OrderedDict(),
+            RelationshipSideChoices.SIDE_SOURCE: {},
+            RelationshipSideChoices.SIDE_DESTINATION: {},
+            RelationshipSideChoices.SIDE_PEER: {},
         }
         for side, relationships in relationships_by_side.items():
             for relationship, queryset in relationships.items():
@@ -180,6 +182,7 @@ class RelationshipModel(models.Model):
                 if resp[side][relationship]["has_many"]:
                     resp[side][relationship]["queryset"] = queryset
                 else:
+                    resp[side][relationship]["url"] = None
                     association = queryset.first()
                     if not association:
                         continue
@@ -187,10 +190,27 @@ class RelationshipModel(models.Model):
                     peer = association.get_peer(self)
 
                     resp[side][relationship]["value"] = peer
-                    resp[side][relationship]["url"] = peer.get_absolute_url()
+                    if hasattr(peer, "get_absolute_url"):
+                        resp[side][relationship]["url"] = peer.get_absolute_url()
+                    else:
+                        logger.warning("Peer object %s has no get_absolute_url() method", peer)
 
         return resp
 
+    def get_relationships_data_basic_fields(self):
+        """
+        Same docstring as get_relationships_data() above except this only returns relationships
+        where advanced_ui==False for displaying in the main object detail tab on the object's page
+        """
+        return self.get_relationships_data(advanced_ui=False)
+
+    def get_relationships_data_advanced_fields(self):
+        """
+        Same docstring as get_relationships_data() above except this only returns relationships
+        where advanced_ui==True for displaying in the 'Advanced' tab on the object's page
+        """
+        return self.get_relationships_data(advanced_ui=True)
+
 
 class RelationshipManager(models.Manager.from_queryset(RestrictedQuerySet)):
     use_in_migrations = True
@@ -275,6 +295,12 @@ class Relationship(BaseModel, ChangeLoggedModel):
         null=True,
         help_text="Queryset filter matching the applicable destination objects of the selected type",
     )
+    advanced_ui = models.BooleanField(
+        default=False,
+        verbose_name="Move to Advanced tab",
+        help_text="Hide this field from the object's primary information tab. "
+        'It will appear in the "Advanced" tab instead.',
+    )
 
     objects = RelationshipManager()
 
@@ -424,7 +450,7 @@ class Relationship(BaseModel, ChangeLoggedModel):
                 for key in filterset.errors:
                     error_messages.append(f"'{key}': " + ", ".join(filterset.errors[key]))
 
-            filterset_params = set(filterset.get_filters().keys())
+            filterset_params = set(filterset.filters.keys())
             for key in filter.keys():
                 if key not in filterset_params:
                     error_messages.append(f"'{key}' is not a valid filter parameter for {model_name} object")
@@ -480,15 +506,16 @@ class Relationship(BaseModel, ChangeLoggedModel):
                 )
 
 
+@extras_features("custom_validators")
 class RelationshipAssociation(BaseModel):
     relationship = models.ForeignKey(to="extras.Relationship", on_delete=models.CASCADE, related_name="associations")
 
     source_type = models.ForeignKey(to=ContentType, on_delete=models.CASCADE, related_name="+")
-    source_id = models.UUIDField()
+    source_id = models.UUIDField(db_index=True)
     source = GenericForeignKey(ct_field="source_type", fk_field="source_id")
 
     destination_type = models.ForeignKey(to=ContentType, on_delete=models.CASCADE, related_name="+")
-    destination_id = models.UUIDField()
+    destination_id = models.UUIDField(db_index=True)
     destination = GenericForeignKey(ct_field="destination_type", fk_field="destination_id")
 
     class Meta:
@@ -501,18 +528,53 @@ class RelationshipAssociation(BaseModel):
         )
 
     def __str__(self):
-        if self.relationship.symmetric:
-            return "{} <-> {} - {}".format(self.source, self.destination, self.relationship)
-        else:
-            return "{} -> {} - {}".format(self.source, self.destination, self.relationship)
+        arrow = "<->" if self.relationship.symmetric else "->"
+        return f"{self.get_source() or 'unknown'} {arrow} {self.get_destination() or 'unknown'} - {self.relationship}"
+
+    def _get_genericforeignkey(self, name):
+        """
+        Backend for get_source and get_destination methods.
+
+        In the case where we have a RelationshipAssociation to a plugin-provided model, but the plugin is
+        not presently installed/enabled, dereferencing the peer GenericForeignKey will throw an AttributeError:
+            AttributeError: 'NoneType' object has no attribute '_base_manager'
+        because ContentType.model_class() returned None unexpectedly.
+
+        This method handles that exception and returns None in such a case.
+        """
+        if name not in ["source", "destination"]:
+            raise RuntimeError(f"Called for unexpected attribute {name}")
+        try:
+            return getattr(self, name)
+        except AttributeError:
+            logger.error(
+                "Unable to locate RelationshipAssociation %s (of type %s). Perhaps a plugin is missing?",
+                name,
+                getattr(self, f"{name}_type"),
+            )
+
+        return None
+
+    def get_source(self):
+        """Accessor for self.source - returns None if the object cannot be located."""
+        return self._get_genericforeignkey("source")
+
+    def get_destination(self):
+        """Accessor for self.destination - returns None if the object cannot be located."""
+        return self._get_genericforeignkey("destination")
 
     def get_peer(self, obj):
+        """
+        Get the object on the opposite side of this RelationshipAssociation from the provided `obj`.
 
-        if obj == self.source:
-            return self.destination
+        If obj is not involved in this RelationshipAssociation, or if the peer object is not locatable, returns None.
+        """
+        if obj == self.get_source():
+            return self.get_destination()
+        elif obj == self.get_destination():
+            return self.get_source()
 
-        elif obj == self.destination:
-            return self.source
+        return None
 
     def clean(self):
 
@@ -538,7 +600,11 @@ class RelationshipAssociation(BaseModel):
             ).exists():
                 raise ValidationError(
                     {
-                        "__all__": f"A {self.relationship} association already exists between {self.source} and {self.destination}"
+                        "__all__": (
+                            f"A {self.relationship} association already exists between "
+                            f"{self.get_source() or self.source_id} and "
+                            f"{self.get_destination() or self.destination_id}"
+                        )
                     }
                 )
 
@@ -555,7 +621,10 @@ class RelationshipAssociation(BaseModel):
             ).exists():
                 raise ValidationError(
                     {
-                        "destination": f"Unable to create more than one {self.relationship} association to {self.destination} (destination)"
+                        "destination": (
+                            f"Unable to create more than one {self.relationship} association to "
+                            f"{self.get_destination() or self.destination_id} (destination)"
+                        )
                     }
                 )
 
@@ -571,7 +640,10 @@ class RelationshipAssociation(BaseModel):
                 ).exists():
                     raise ValidationError(
                         {
-                            "source": f"Unable to create more than one {self.relationship} association to {self.source} (source)"
+                            "source": (
+                                f"Unable to create more than one {self.relationship} association from "
+                                f"{self.get_source() or self.source_id} (source)"
+                            )
                         }
                     )
 
@@ -584,7 +656,10 @@ class RelationshipAssociation(BaseModel):
                 ).exists():
                     raise ValidationError(
                         {
-                            "source": f"Unable to create more than one {self.relationship} association involving {self.source} (peer)"
+                            "source": (
+                                f"Unable to create more than one {self.relationship} association involving "
+                                f"{self.get_source() or self.source_id} (peer)"
+                            )
                         }
                     )
                 if RelationshipAssociation.objects.filter(
@@ -593,6 +668,9 @@ class RelationshipAssociation(BaseModel):
                 ).exists():
                     raise ValidationError(
                         {
-                            "destination": f"Unable to create more than one {self.relationship} association involving {self.destination} (peer)"
+                            "destination": (
+                                f"Unable to create more than one {self.relationship} association involving "
+                                f"{self.get_destination() or self.destination_id} (peer)"
+                            )
                         }
                     )
