@@ -23,37 +23,41 @@ from jsonschema.validators import Draft7Validator
 from nautobot.core.views import generic
 from nautobot.dcim.models import Device
 from nautobot.dcim.tables import DeviceTable
-from nautobot.extras.utils import get_worker_count
+from nautobot.extras.utils import get_job_content_type, get_worker_count
 from nautobot.utilities.paginator import EnhancedPaginator, get_paginate_count
 from nautobot.utilities.forms import restrict_form_fields
 from nautobot.utilities.utils import (
     copy_safe_request,
     count_related,
+    get_table_for_model,
     prepare_cloned_fields,
     shallow_compare_dict,
 )
 from nautobot.utilities.tables import ButtonsColumn
-from nautobot.utilities.views import ContentTypePermissionRequiredMixin
+from nautobot.utilities.views import ObjectPermissionRequiredMixin
 from nautobot.utilities.utils import normalize_querydict
 from nautobot.virtualization.models import VirtualMachine
 from nautobot.virtualization.tables import VirtualMachineTable
 from . import filters, forms, tables
 from .choices import JobExecutionType, JobResultStatusChoices
 from .datasources import (
-    get_datasource_contents,
+    enqueue_git_repository_diff_origin_and_local,
     enqueue_pull_git_repository_and_refresh_data,
+    get_datasource_contents,
 )
-from .jobs import get_job, get_jobs, run_job, Job
+from .jobs import get_job, run_job, Job as JobClass
 from .models import (
     ComputedField,
     ConfigContext,
     ConfigContextSchema,
     CustomField,
     CustomLink,
+    DynamicGroup,
     ExportTemplate,
     GitRepository,
     GraphQLQuery,
     ImageAttachment,
+    Job as JobModel,
     ObjectChange,
     JobResult,
     Relationship,
@@ -70,6 +74,9 @@ from .models import (
 from .registry import registry
 
 
+logger = logging.getLogger(__name__)
+
+
 #
 # Computed Fields
 #
@@ -216,7 +223,6 @@ class ConfigContextSchemaView(generic.ObjectView):
 
         return {
             "format": format,
-            "active_tab": "configcontextschema",
         }
 
 
@@ -360,7 +366,6 @@ class CustomFieldEditView(generic.ObjectEditView):
         return ctx
 
     def post(self, request, *args, **kwargs):
-        logger = logging.getLogger("nautobot.views.CustomFieldEditView")
         obj = self.alter_obj(self.get_object(kwargs), request, args, kwargs)
         form = self.model_form(data=request.POST, files=request.FILES, instance=obj)
         restrict_form_fields(form, request.user)
@@ -482,6 +487,162 @@ class CustomLinkBulkDeleteView(generic.BulkDeleteView):
     table = tables.CustomLinkTable
 
 
+#
+# Dynamic Groups
+#
+
+
+class DynamicGroupListView(generic.ObjectListView):
+    queryset = DynamicGroup.objects.all()
+    table = tables.DynamicGroupTable
+    filterset = filters.DynamicGroupFilterSet
+    filterset_form = forms.DynamicGroupFilterForm
+    action_buttons = ("add",)
+
+
+class DynamicGroupView(generic.ObjectView):
+    queryset = DynamicGroup.objects.all()
+
+    def get_extra_context(self, request, instance):
+        context = super().get_extra_context(request, instance)
+        model = instance.content_type.model_class()
+        table_class = get_table_for_model(model)
+
+        if table_class is not None:
+            members = instance.get_queryset()
+            members_table = table_class(members, orderable=False)
+
+            # Paginate the members table.
+            paginate = {
+                "paginator_class": EnhancedPaginator,
+                "per_page": get_paginate_count(request),
+            }
+            RequestConfig(request, paginate).configure(members_table)
+
+            context["members_table"] = members_table
+
+        return context
+
+
+class DynamicGroupEditView(generic.ObjectEditView):
+    queryset = DynamicGroup.objects.all()
+    model_form = forms.DynamicGroupForm
+    template_name = "extras/dynamicgroup_edit.html"
+
+    def get_extra_context(self, request, instance):
+        ctx = super().get_extra_context(request, instance)
+
+        filterform_class = instance.generate_filter_form()
+
+        if filterform_class is None:
+            # FIXME(jathan): There is currently an edge case here that needs to be addressed:
+            # `AttributeError: 'NoneType' object has no attribute 'is_valid'`
+            # See: https://sentry.io/share/issue/fb41c6afb40248f6931021574bc38a0d/
+            filter_form = None
+        elif request.POST:
+            filter_form = filterform_class(data=request.POST)
+        else:
+            initial = instance.get_initial()
+            filter_form = filterform_class(initial=initial)
+
+        ctx["filter_form"] = filter_form
+
+        return ctx
+
+    def post(self, request, *args, **kwargs):
+        obj = self.alter_obj(self.get_object(kwargs), request, args, kwargs)
+        form = self.model_form(data=request.POST, files=request.FILES, instance=obj)
+        restrict_form_fields(form, request.user)
+
+        if form.is_valid():
+            logger.debug("Form validation was successful")
+
+            try:
+                with transaction.atomic():
+                    object_created = not form.instance.present_in_database
+                    # Obtain the instance, but do not yet `save()` it to the database.
+                    obj = form.save(commit=False)
+
+                    # Process the filter form and save the query filters to `obj.filter`.
+                    ctx = self.get_extra_context(request, obj)
+                    filter_form = ctx["filter_form"]
+                    if filter_form.is_valid():
+                        obj.set_filter(filter_form.cleaned_data)
+                    else:
+                        raise RuntimeError(filter_form.errors)
+
+                    # After filters have been set, now we save the object to the database.
+                    obj.save()
+                    # Check that the new object conforms with any assigned object-level permissions
+                    self.queryset.get(pk=obj.pk)
+
+                msg = "{} {}".format(
+                    "Created" if object_created else "Modified",
+                    self.queryset.model._meta.verbose_name,
+                )
+                logger.info(f"{msg} {obj} (PK: {obj.pk})")
+                if hasattr(obj, "get_absolute_url"):
+                    msg = '{} <a href="{}">{}</a>'.format(msg, obj.get_absolute_url(), escape(obj))
+                else:
+                    msg = "{} {}".format(msg, escape(obj))
+                messages.success(request, mark_safe(msg))
+
+                if "_addanother" in request.POST:
+
+                    # If the object has clone_fields, pre-populate a new instance of the form
+                    if hasattr(obj, "clone_fields"):
+                        url = "{}?{}".format(request.path, prepare_cloned_fields(obj))
+                        return redirect(url)
+
+                    return redirect(request.get_full_path())
+
+                return_url = form.cleaned_data.get("return_url")
+                if return_url is not None and is_safe_url(url=return_url, allowed_hosts=request.get_host()):
+                    return redirect(return_url)
+                else:
+                    return redirect(self.get_return_url(request, obj))
+
+            except ObjectDoesNotExist:
+                msg = "Object save failed due to object-level permissions violation."
+                logger.debug(msg)
+                form.add_error(None, msg)
+            except RuntimeError:
+                msg = "Errors encountered when saving Dynamic Group associations. See below."
+                logger.debug(msg)
+                form.add_error(None, msg)
+            except ProtectedError as err:
+                # e.g. Trying to delete a something that is in use.
+                protected_obj, err_msg = err.args
+                msg = f"{protected_obj.value}: {err_msg} Please cancel this edit and start again."
+                logger.debug(msg)
+                form.add_error(None, msg)
+
+        else:
+            logger.debug("Form validation failed")
+
+        return render(
+            request,
+            self.template_name,
+            {
+                "obj": obj,
+                "obj_type": self.queryset.model._meta.verbose_name,
+                "form": form,
+                "return_url": self.get_return_url(request, obj),
+                "editing": obj.present_in_database,
+                **self.get_extra_context(request, obj),
+            },
+        )
+
+
+class DynamicGroupDeleteView(generic.ObjectDeleteView):
+    queryset = DynamicGroup.objects.all()
+
+
+class DynamicGroupBulkDeleteView(generic.BulkDeleteView):
+    queryset = DynamicGroup.objects.all()
+    table = tables.DynamicGroupTable
+
+
 #
 # Export Templates
 #
@@ -555,6 +716,7 @@ class GitRepositoryView(generic.ObjectView):
 class GitRepositoryEditView(generic.ObjectEditView):
     queryset = GitRepository.objects.all()
     model_form = forms.GitRepositoryForm
+    template_name = "extras/gitrepository_object_edit.html"
 
     def alter_obj(self, obj, request, url_args, url_kwargs):
         # A GitRepository needs to know the originating request when it's saved so that it can enqueue using it
@@ -612,20 +774,36 @@ class GitRepositoryBulkDeleteView(generic.BulkDeleteView):
         }
 
 
+def check_and_call_git_repository_function(request, slug, func):
+    """Helper for checking Git permissions and worker availability, then calling provided function if all is well
+    Args:
+        request: request object.
+        slug (str): GitRepository slug value.
+        func (function): Enqueue git repo function.
+    Returns:
+        HttpResponseForbidden or a redirect
+    """
+    if not request.user.has_perm("extras.change_gitrepository"):
+        return HttpResponseForbidden()
+
+    # Allow execution only if a worker process is running.
+    if not get_worker_count(request):
+        messages.error(request, "Unable to run job: Celery worker process not running.")
+    else:
+        repository = get_object_or_404(GitRepository, slug=slug)
+        func(repository, request)
+
+    return redirect("extras:gitrepository_result", slug=slug)
+
+
 class GitRepositorySyncView(View):
     def post(self, request, slug):
-        if not request.user.has_perm("extras.change_gitrepository"):
-            return HttpResponseForbidden()
+        return check_and_call_git_repository_function(request, slug, enqueue_pull_git_repository_and_refresh_data)
 
-        repository = get_object_or_404(GitRepository.objects.all(), slug=slug)
 
-        # Allow execution only if a worker process is running.
-        if not get_worker_count(request):
-            messages.error(request, "Unable to run job: Celery worker process not running.")
-        else:
-            enqueue_pull_git_repository_and_refresh_data(repository, request)
-
-        return redirect("extras:gitrepository_result", slug=slug)
+class GitRepositoryDryRunView(View):
+    def post(self, request, slug):
+        return check_and_call_git_repository_function(request, slug, enqueue_git_repository_diff_origin_and_local)
 
 
 class GitRepositoryResultView(generic.ObjectView):
@@ -718,106 +896,99 @@ class ImageAttachmentDeleteView(generic.ObjectDeleteView):
 #
 
 
-class JobListView(ContentTypePermissionRequiredMixin, View):
+class JobListView(generic.ObjectListView):
     """
     Retrieve all of the available jobs from disk and the recorded JobResult (if any) for each.
     """
 
-    def get_required_permission(self):
-        return "extras.view_job"
+    queryset = JobModel.objects.all()
+    table = tables.JobTable
+    filterset = filters.JobFilterSet
+    filterset_form = forms.JobFilterForm
+    action_buttons = ()
+    template_name = "extras/job_list.html"
 
-    def get(self, request):
-        jobs_dict = get_jobs()
-        job_content_type = ContentType.objects.get(app_label="extras", model="job")
-        # Get the newest results for each job name
-        results = {
-            r.name: r
-            for r in JobResult.objects.filter(
-                obj_type=job_content_type,
-                status__in=JobResultStatusChoices.TERMINAL_STATE_CHOICES,
-            )
-            .order_by("completed")
-            .defer("data")
+    def alter_queryset(self, request):
+        queryset = super().alter_queryset(request)
+        # Default to hiding "hidden" and non-installed jobs
+        if "hidden" not in request.GET:
+            queryset = queryset.filter(hidden=False)
+        if "installed" not in request.GET:
+            queryset = queryset.filter(installed=True)
+        queryset = queryset.prefetch_related("results")
+        return queryset
+
+    def extra_context(self):
+        return {
+            "table_inc_template": "extras/inc/job_table.html",
         }
 
-        # get_jobs() gives us a nested dict {grouping: {module: {"name": name, "jobs": [job, job, job]}}}
-        # But for presentation to the user we want to flatten this to {module_name: [job, job, job]}
 
-        modules_dict = {}
-        for grouping, modules in jobs_dict.items():
-            for module, entry in modules.items():
-                module_jobs = modules_dict.get(entry["name"], [])
-                for job_class in entry["jobs"].values():
-                    job = job_class()
-                    job.result = results.get(job.class_path, None)
-                    module_jobs.append(job)
-                if module_jobs:
-                    # TODO: should we sort module_jobs by job name? Currently they're in source code order
-                    modules_dict[entry["name"]] = module_jobs
-
-        return render(
-            request,
-            "extras/job_list.html",
-            {
-                # Order the jobs listing by case-insensitive sorting of the module human-readable name
-                "jobs": sorted(modules_dict.items(), key=lambda kvpair: kvpair[0].lower()),
-            },
-        )
-
-
-class JobView(ContentTypePermissionRequiredMixin, View):
+# 2.0 TODO: this should really be "JobRunView"
+class JobView(ObjectPermissionRequiredMixin, View):
     """
     View the parameters of a Job and enqueue it if desired.
     """
 
+    queryset = JobModel.objects.all()
+
     def get_required_permission(self):
-        return "extras.view_job"
+        return "extras.run_job"
 
-    def _get_job(self, class_path):
-        job_class = get_job(class_path)
-        if job_class is None:
-            raise Http404
-        return job_class
+    def _get_job_model_or_404(self, class_path=None, slug=None):
+        """Helper function for get() and post()."""
+        if class_path:
+            try:
+                job_model = self.queryset.get_for_class_path(class_path)
+            except JobModel.DoesNotExist:
+                raise Http404
+        else:
+            job_model = get_object_or_404(self.queryset, slug=slug)
 
-    def get(self, request, class_path):
-        job_class = self._get_job(class_path)
-        job = job_class()
-        grouping, module, class_name = class_path.split("/", 2)
+        return job_model
+
+    def get(self, request, class_path=None, slug=None):
+        job_model = self._get_job_model_or_404(class_path, slug)
+
+        try:
+            job_form = job_model.job_class().as_form(initial=normalize_querydict(request.GET))
+        except RuntimeError as err:
+            messages.error(request, f"Unable to run or schedule '{job_model}': {err}")
+            return redirect("extras:job_list")
 
-        job_form = job.as_form(initial=normalize_querydict(request.GET))
         schedule_form = forms.JobScheduleForm(initial=normalize_querydict(request.GET))
 
         return render(
             request,
-            "extras/job.html",
+            "extras/job.html",  # 2.0 TODO: extras/job_submission.html
             {
-                "grouping": grouping,
-                "module": module,
-                "job": job,
+                "job_model": job_model,
                 "job_form": job_form,
                 "schedule_form": schedule_form,
             },
         )
 
-    def post(self, request, class_path):
-        if not request.user.has_perm("extras.run_job"):
-            return HttpResponseForbidden()
+    def post(self, request, class_path=None, slug=None):
+        job_model = self._get_job_model_or_404(class_path, slug)
 
-        job_class = self._get_job(class_path)
-        job = job_class()
-        grouping, module, class_name = class_path.split("/", 2)
-        job_form = job.as_form(request.POST, request.FILES)
+        job_form = (
+            job_model.job_class().as_form(request.POST, request.FILES) if job_model.job_class is not None else None
+        )
         schedule_form = forms.JobScheduleForm(request.POST)
 
-        # Allow execution only if a worker process is running.
+        # Allow execution only if a worker process is running and the job is runnable.
         if not get_worker_count(request):
-            messages.error(request, "Unable to run job: Celery worker process not running.")
-        elif job_form.is_valid() and schedule_form.is_valid():
+            messages.error(request, "Unable to run or schedule job: Celery worker process not running.")
+        elif not job_model.installed or job_model.job_class is None:
+            messages.error(request, "Unable to run or schedule job: Job is not presently installed.")
+        elif not job_model.enabled:
+            messages.error(request, "Unable to run or schedule job: Job is not enabled to be run.")
+        elif job_form is not None and job_form.is_valid() and schedule_form.is_valid():
             # Run the job. A new JobResult is created.
             commit = job_form.cleaned_data.pop("_commit")
             schedule_type = schedule_form.cleaned_data["_schedule_type"]
 
-            if job_class.approval_required or schedule_type in JobExecutionType.SCHEDULE_CHOICES:
+            if job_model.approval_required or schedule_type in JobExecutionType.SCHEDULE_CHOICES:
 
                 if schedule_type in JobExecutionType.SCHEDULE_CHOICES:
                     # Schedule the job instead of running it now
@@ -831,32 +1002,33 @@ class JobView(ContentTypePermissionRequiredMixin, View):
                     # when approval is granted, the task is immediately due for execution.
                     schedule_type = JobExecutionType.TYPE_FUTURE
                     schedule_datetime = datetime.now()
-                    schedule_name = f"{job.name} - {schedule_datetime}"
+                    schedule_name = f"{job_model} - {schedule_datetime}"
 
                 job_kwargs = {
-                    "data": job_class.serialize_data(job_form.cleaned_data),
+                    "data": job_model.job_class.serialize_data(job_form.cleaned_data),
                     "request": copy_safe_request(request),
                     "user": request.user.pk,
                     "commit": commit,
-                    "name": job.class_path,
+                    "name": job_model.class_path,
                 }
 
                 scheduled_job = ScheduledJob(
                     name=schedule_name,
                     task="nautobot.extras.jobs.scheduled_job_handler",
-                    job_class=job.class_path,
+                    job_class=job_model.class_path,
+                    job_model=job_model,
                     start_time=schedule_datetime,
-                    description=f"Nautobot job {schedule_name} scheduled by {request.user} on {schedule_datetime}",
+                    description=f"Nautobot job {schedule_name} scheduled by {request.user} for {schedule_datetime}",
                     kwargs=job_kwargs,
                     interval=schedule_type,
                     one_off=schedule_type == JobExecutionType.TYPE_FUTURE,
                     user=request.user,
-                    approval_required=job_class.approval_required,
+                    approval_required=job_model.approval_required,
                 )
                 scheduled_job.kwargs["scheduled_job_pk"] = scheduled_job.pk
                 scheduled_job.save()
 
-                if job_class.approval_required:
+                if job_model.approval_required:
                     messages.success(request, f"Job {schedule_name} successfully submitted for approval")
                     return redirect("extras:scheduledjob_approval_queue_list")
                 else:
@@ -865,13 +1037,13 @@ class JobView(ContentTypePermissionRequiredMixin, View):
 
             else:
                 # Enqueue job for immediate execution
-                job_content_type = ContentType.objects.get(app_label="extras", model="job")
+                job_content_type = get_job_content_type()
                 job_result = JobResult.enqueue_job(
                     run_job,
-                    job.class_path,
+                    job_model.class_path,
                     job_content_type,
                     request.user,
-                    data=job_class.serialize_data(job_form.cleaned_data),
+                    data=job_model.job_class.serialize_data(job_form.cleaned_data),
                     request=copy_safe_request(request),
                     commit=commit,
                 )
@@ -882,16 +1054,30 @@ class JobView(ContentTypePermissionRequiredMixin, View):
             request,
             "extras/job.html",
             {
-                "grouping": grouping,
-                "module": module,
-                "job": job,
+                "job_model": job_model,
                 "job_form": job_form,
                 "schedule_form": schedule_form,
             },
         )
 
 
-class JobApprovalRequestView(ContentTypePermissionRequiredMixin, View):
+# 2.0 TODO: this should really be "JobView"
+class JobDetailView(generic.ObjectView):
+    queryset = JobModel.objects.all()
+    template_name = "extras/job_detail.html"
+
+
+class JobEditView(generic.ObjectEditView):
+    queryset = JobModel.objects.all()
+    model_form = forms.JobEditForm
+    template_name = "extras/job_edit.html"
+
+
+class JobDeleteView(generic.ObjectDeleteView):
+    queryset = JobModel.objects.all()
+
+
+class JobApprovalRequestView(generic.ObjectView):
     """
     This view handles requests to view and approve a Job execution request.
     It renders the Job's form in much the same way as `JobView` except all
@@ -899,10 +1085,36 @@ class JobApprovalRequestView(ContentTypePermissionRequiredMixin, View):
     job's execution, rather than initial job form input.
     """
 
-    def get_required_permission(self):
-        return "extras.view_job"
+    queryset = ScheduledJob.objects.filter(task="nautobot.extras.jobs.scheduled_job_handler").needs_approved()
+    template_name = "extras/job_approval_request.html"
+    additional_permissions = ("extras.view_job",)
 
-    def post(self, request, scheduled_job):
+    def get_extra_context(self, request, instance):
+        """
+        Render the job form with data from the scheduled_job instance, but mark all fields as disabled.
+        We don't care to actually get any data back from the form as we will not ever change it.
+        Instead, we offer the user three submit buttons, dry-run, approve, and deny, which we act upon in the post.
+        """
+        job_model = instance.job_model
+        if job_model is not None:
+            job_class = job_model.job_class
+        else:
+            # 2.0 TODO: remove this fallback?
+            job_class = get_job(instance.job_class)
+
+        if job_class is not None:
+            # Render the form with all fields disabled
+            initial = instance.kwargs.get("data", {})
+            initial["_commit"] = instance.kwargs.get("commit", True)
+            job_form = job_class().as_form(initial=initial, approval_view=True)
+        else:
+            job_form = None
+
+        return {
+            "job_form": job_form,
+        }
+
+    def post(self, request, pk):
         """
         Act upon one of the 3 submit button actions from the user.
 
@@ -910,20 +1122,7 @@ class JobApprovalRequestView(ContentTypePermissionRequiredMixin, View):
         deny will delete the scheduled_job instance
         approve will mark the scheduled_job as approved, allowing the schedular to schedule the job execution task
         """
-        if not request.user.has_perm("extras.run_job"):
-            return HttpResponseForbidden()
-
-        scheduled_job = get_object_or_404(ScheduledJob, pk=scheduled_job)
-        job_class = get_job(scheduled_job.job_class)
-        if job_class is None:
-            raise Http404
-        job = job_class()
-        grouping, module, class_name = job_class.class_path.split("/", 2)
-
-        # Render the form will all fields disabled
-        initial = scheduled_job.kwargs.get("data", {})
-        initial["_commit"] = scheduled_job.kwargs.get("commit", True)
-        job_form = job.as_form(initial=initial, approval_view=True)
+        scheduled_job = get_object_or_404(ScheduledJob, pk=pk)
 
         post_data = request.POST
 
@@ -932,37 +1131,62 @@ class JobApprovalRequestView(ContentTypePermissionRequiredMixin, View):
         force_approve = "_force_approve" in post_data
         dry_run = "_dry_run" in post_data
 
+        job_model = scheduled_job.job_model
+
         if dry_run:
-            # Immediately enqueue the job with commit=False and send the user to the normal JobResult view
-            job_content_type = ContentType.objects.get(app_label="extras", model="job")
-            job_result = JobResult.enqueue_job(
-                run_job,
-                job.class_path,
-                job_content_type,
-                scheduled_job.user,
-                request.user,
-                data=job_class.serialize_data(initial),
-                request=copy_safe_request(request),
-                commit=False,  # force a dry-run
-            )
-
-            return redirect("extras:job_jobresult", pk=job_result.pk)
-        elif deny:
-            # Delete the scheduled_job instance
-            scheduled_job.delete()
-            if request.user == scheduled_job.user:
-                messages.error(request, f"Approval request for {scheduled_job.name} was revoked")
+            # To dry-run a job, a user needs the same permissions that would be needed to run the job directly
+            if job_model is None:
+                messages.error(request, "There is no job associated with this request? Cannot run it!")
+            elif not job_model.runnable:
+                messages.error(request, "This job cannot be run at this time")
+            elif not JobModel.objects.check_perms(self.request.user, instance=job_model, action="run"):
+                messages.error(request, "You do not have permission to run this job")
             else:
-                messages.error(request, f"Approval of {scheduled_job.name} was denied")
+                # Immediately enqueue the job with commit=False and send the user to the normal JobResult view
+                job_content_type = get_job_content_type()
+                initial = scheduled_job.kwargs.get("data", {})
+                initial["_commit"] = False
+                job_result = JobResult.enqueue_job(
+                    run_job,
+                    job_model.job_class.class_path,
+                    job_content_type,
+                    request.user,
+                    data=job_model.job_class.serialize_data(initial),
+                    request=copy_safe_request(request),
+                    commit=False,  # force a dry-run
+                )
 
-            return redirect("extras:scheduledjob_approval_queue_list")
+                return redirect("extras:job_jobresult", pk=job_result.pk)
+        elif deny:
+            if not (
+                self.queryset.check_perms(request.user, instance=scheduled_job, action="delete")
+                and job_model is not None
+                and JobModel.objects.check_perms(request.user, instance=job_model, action="approve")
+            ):
+                messages.error(request, "You do not have permission to deny this request.")
+            else:
+                # Delete the scheduled_job instance
+                scheduled_job.delete()
+                if request.user == scheduled_job.user:
+                    messages.error(request, f"Approval request for {scheduled_job.name} was revoked")
+                else:
+                    messages.error(request, f"Approval of {scheduled_job.name} was denied")
+
+                return redirect("extras:scheduledjob_approval_queue_list")
 
         elif approve or force_approve:
-            # Mark the scheduled_job as approved, allowing the schedular to schedule the job execution task
-            if request.user == scheduled_job.user:
+            if job_model is None:
+                messages.error(request, "There is no job associated with this request? Cannot run it!")
+            elif not (
+                self.queryset.check_perms(request.user, instance=scheduled_job, action="change")
+                and JobModel.objects.check_perms(request.user, instance=job_model, action="approve")
+            ):
+                messages.error(request, "You do not have permission to approve this request.")
+            elif request.user == scheduled_job.user:
                 # The requestor *cannot* approve their own job
                 messages.error(request, "You cannot approve your own job request!")
             else:
+                # Mark the scheduled_job as approved, allowing the schedular to schedule the job execution task
                 if scheduled_job.one_off and scheduled_job.start_time < timezone.now() and not force_approve:
                     return render(request, "extras/job_approval_confirmation.html", {"scheduled_job": scheduled_job})
                 scheduled_job.approved_by_user = request.user
@@ -971,47 +1195,14 @@ class JobApprovalRequestView(ContentTypePermissionRequiredMixin, View):
 
                 messages.success(request, f"{scheduled_job.name} was approved and will now begin execution")
 
-            return redirect("extras:scheduledjob_approval_queue_list")
+                return redirect("extras:scheduledjob_approval_queue_list")
 
         return render(
             request,
-            "extras/job_approval_request.html",
+            self.get_template_name(),
             {
-                "grouping": grouping,
-                "module": module,
-                "job": job,
-                "job_form": job_form,
-                "scheduled_job": scheduled_job,
-            },
-        )
-
-    def get(self, request, scheduled_job):
-        """
-        Render the job form with data from the scheduled_job instance, but mark all fields as disabled.
-        We don't care to actually get any data back from the form as we will not ever change it.
-        Instead, we offer the user three submit buttons, dry-run, approve, and deny, which we act upon in the post.
-        """
-        scheduled_job = get_object_or_404(ScheduledJob, pk=scheduled_job)
-        job_class = get_job(scheduled_job.job_class)
-        if job_class is None:
-            raise Http404
-        job = job_class()
-        grouping, module, class_name = job_class.class_path.split("/", 2)
-
-        # Render the form will all fields disabled
-        initial = scheduled_job.kwargs.get("data", {})
-        initial["_commit"] = scheduled_job.kwargs.get("commit", True)
-        job_form = job.as_form(initial=initial, approval_view=True)
-
-        return render(
-            request,
-            "extras/job_approval_request.html",
-            {
-                "grouping": grouping,
-                "module": module,
-                "job": job,
-                "job_form": job_form,
-                "scheduled_job": scheduled_job,
+                "object": scheduled_job,
+                **self.get_extra_context(request, scheduled_job),
             },
         )
 
@@ -1074,7 +1265,7 @@ class JobResultListView(generic.ObjectListView):
     List JobResults
     """
 
-    queryset = JobResult.objects.all()
+    queryset = JobResult.objects.prefetch_related("job_model", "logs", "obj_type", "user")
     filterset = filters.JobResultFilterSet
     filterset_form = forms.JobResultFilterForm
     table = tables.JobResultTable
@@ -1100,15 +1291,19 @@ class JobResultView(generic.ObjectView):
 
     def get_extra_context(self, request, instance):
         associated_record = None
-        job = None
+        job_class = None
+        if instance.job_model is not None:
+            job_class = instance.job_model.job_class
+        # 2.0 TODO: remove JobResult.related_object entirely
         related_object = instance.related_object
-        if inspect.isclass(related_object) and issubclass(related_object, Job):
-            job = related_object()
+        if inspect.isclass(related_object) and issubclass(related_object, JobClass):
+            if job_class is None:
+                job_class = related_object
         elif related_object:
             associated_record = related_object
 
         return {
-            "job": job,
+            "job": job_class,
             "associated_record": associated_record,
             "result": instance,
         }
@@ -1141,6 +1336,26 @@ class ObjectChangeListView(generic.ObjectListView):
     template_name = "extras/objectchange_list.html"
     action_buttons = ("export",)
 
+    # TODO: Remove this remapping in 2.0 as it is addressing a potentially breaking change
+    def get(self, request, **kwargs):
+
+        # Remappings below allow previous queries of time_before and time_after to use
+        # newer methods specifying the lookup method.
+
+        # They will only use the previous arguments if the newer ones are undefined
+
+        if request.GET.get("time_after") and request.GET.get("time__gte") is None:
+            request.GET._mutable = True
+            request.GET.update({"time__gte": request.GET.get("time_after")})
+            request.GET._mutable = False
+
+        if request.GET.get("time_before") and request.GET.get("time__lte") is None:
+            request.GET._mutable = True
+            request.GET.update({"time__lte": request.GET.get("time_before")})
+            request.GET._mutable = False
+
+        return super().get(request=request, **kwargs)
+
 
 class ObjectChangeView(generic.ObjectView):
     queryset = ObjectChange.objects.all()
@@ -1185,7 +1400,6 @@ class ObjectChangeView(generic.ObjectView):
 class ObjectChangeLogView(View):
     """
     Present a history of changes made to a particular object.
-
     base_template: The name of the template to extend. If not provided, "<app>/<model>.html" will be used.
     """
 
@@ -1396,7 +1610,6 @@ class SecretsGroupEditView(generic.ObjectEditView):
         return ctx
 
     def post(self, request, *args, **kwargs):
-        logger = logging.getLogger("nautobot.views.SecretsGroupEditView")
         obj = self.alter_obj(self.get_object(kwargs), request, args, kwargs)
         form = self.model_form(data=request.POST, files=request.FILES, instance=obj)
         restrict_form_fields(form, request.user)
@@ -1577,6 +1790,7 @@ class TagView(generic.ObjectView):
         return {
             "items_count": tagged_items.count(),
             "items_table": items_table,
+            "content_types": instance.content_types.order_by("app_label", "model"),
         }
 
 
