@@ -1,10 +1,12 @@
 from unittest import skipIf
 
 import netaddr
+from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.db import connection
 from django.test import TestCase, override_settings
 
+from nautobot.dcim.models import Device, DeviceRole, DeviceType, Interface, Manufacturer, Site
 from nautobot.extras.models import Status
 from nautobot.ipam.choices import IPAddressRoleChoices
 from nautobot.ipam.models import Aggregate, IPAddress, Prefix, RIR, VLAN, VLANGroup, VRF
@@ -397,6 +399,85 @@ class TestIPAddress(TestCase):
             role=IPAddressRoleChoices.ROLE_VIP,
         )
 
+    def test_multiple_nat_outside(self):
+        """
+        Test suite to test supporing multiple nat_inside related fields.
+
+        Includes tests for legacy getter/setter for nat_outside.
+        """
+
+        # Setup mimicked legacy data model relationships: 1-to-1
+        nat_inside = IPAddress.objects.create(address=netaddr.IPNetwork("192.168.0.1/24"))
+        nat_outside1 = IPAddress.objects.create(address=netaddr.IPNetwork("192.0.2.1/24"), nat_inside=nat_inside)
+        nat_inside.refresh_from_db()
+
+        # Assert legacy getter behaves as expected for backwards compatibility, and that FK relationship works
+        self.assertEqual(nat_inside.nat_outside, nat_outside1)
+        self.assertEqual(nat_inside.nat_outside_list.count(), 1)
+        self.assertEqual(nat_inside.nat_outside_list.first(), nat_outside1)
+
+        # Create unassigned IPAddress
+        nat_outside2 = IPAddress.objects.create(address=netaddr.IPNetwork("192.0.2.2/24"))
+        nat_inside.nat_outside = nat_outside2
+        nat_inside.refresh_from_db()
+
+        # Test legacy setter behaves as expected for backwards compatibility and that previous FK relationship removed
+        self.assertEqual(nat_inside.nat_outside, nat_outside2)
+        self.assertEqual(nat_inside.nat_outside_list.count(), 1)
+        self.assertEqual(nat_inside.nat_outside_list.first(), nat_outside2)
+
+        # Create IPAddress with nat_inside assigned, setting up current 1-to-many relationship
+        nat_outside3 = IPAddress.objects.create(address=netaddr.IPNetwork("192.0.2.3/24"), nat_inside=nat_inside)
+        nat_inside.refresh_from_db()
+
+        # Now ensure safeguards are in place when using legacy methods with 1-to-many relationships
+        with self.assertRaises(IPAddress.NATOutsideMultipleObjectsReturned):
+            nat_inside.nat_outside
+        with self.assertRaises(IPAddress.NATOutsideMultipleObjectsReturned):
+            nat_inside.nat_outside = nat_outside1
+
+        # Assert FK relationship behaves as expected
+        self.assertEqual(nat_inside.nat_outside_list.count(), 2)
+        self.assertEqual(nat_inside.nat_outside_list.first(), nat_outside2)
+        self.assertEqual(nat_inside.nat_outside_list.last(), nat_outside3)
+
+    @override_settings(ENFORCE_GLOBAL_UNIQUE=True)
+    def test_not_null_assigned_object_type_and_null_assigned_object_id(self):
+        site = Site.objects.create(name="Test Site 1", slug="test-site-1")
+        manufacturer = Manufacturer.objects.create(name="Test Manufacturer 1", slug="test-manufacturer-1")
+        devicetype = DeviceType.objects.create(
+            manufacturer=manufacturer,
+            model="Test Device Type 1",
+            slug="test-device-type-1",
+        )
+        devicerole = DeviceRole.objects.create(name="Test Device Role 1", slug="test-device-role-1", color="ff0000")
+        device_status = Status.objects.get_for_model(Device).get(slug="active")
+        device = Device.objects.create(
+            device_type=devicetype,
+            device_role=devicerole,
+            name="TestDevice1",
+            site=site,
+            status=device_status,
+        )
+        interface = Interface.objects.create(device=device, name="eth0")
+        ipaddress_1 = IPAddress(
+            address=netaddr.IPNetwork("192.0.2.1/24"),
+            role=IPAddressRoleChoices.ROLE_VIP,
+            assigned_object_id=interface.id,
+        )
+
+        self.assertRaises(ValidationError, ipaddress_1.clean)
+
+        # Test IPAddress.clean() raises no exception if assigned_object_id and assigned_object_type
+        # are both provided
+        ipaddress_2 = IPAddress(
+            address=netaddr.IPNetwork("192.0.2.1/24"),
+            role=IPAddressRoleChoices.ROLE_VIP,
+            assigned_object_id=interface.id,
+            assigned_object_type=ContentType.objects.get_for_model(Interface),
+        )
+        self.assertIsNone(ipaddress_2.clean())
+
 
 class TestVLANGroup(TestCase):
     def test_get_next_available_vid(self):
