@@ -118,13 +118,20 @@ class APISelect(SelectWithDisabled):
     """
     A select widget populated via an API call
 
-    :param api_url: API endpoint URL. Required if not set automatically by the parent field.
+    Args:
+        api_url: API endpoint URL. Required if not set automatically by the parent field.
+        api_version: API version.
     """
 
-    def __init__(self, api_url=None, full=False, *args, **kwargs):
+    def __init__(self, api_url=None, full=False, api_version=None, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
         self.attrs["class"] = "nautobot-select2-api"
+
+        if api_version:
+            # Set Request Accept Header api-version e.g Accept: application/json; version=1.2
+            self.attrs["data-api-version"] = api_version
+
         if api_url:
             # Prefix the URL w/ the script prefix (e.g. `/nautobot`)
             self.attrs["data-url"] = urljoin(get_script_prefix(), api_url.lstrip("/"))
@@ -139,12 +146,12 @@ class APISelect(SelectWithDisabled):
         key = f"data-query-param-{name}"
 
         values = json.loads(self.attrs.get(key, "[]"))
-        if type(value) in (list, tuple):
+        if isinstance(value, (list, tuple)):
             values.extend([str(v) for v in value])
         else:
             values.append(str(value))
 
-        self.attrs[key] = json.dumps(values)
+        self.attrs[key] = json.dumps(values, ensure_ascii=False)
 
 
 class APISelectMultiple(APISelect, forms.SelectMultiple):
